@@ -3,10 +3,8 @@
     
 """
 
-try:
-    import Cookie as cookies
-except ImportError:
-    import http.cookies
+from Cookie import SimpleCookie
+from datetime import datetime
 
 from BaseHTTPServer import BaseHTTPRequestHandler
 from urlparse import parse_qs
@@ -21,12 +19,13 @@ status_codes = BaseHTTPRequestHandler.responses
 
 '''________________________________helpers___________________________________'''
 
-def _headers(headers, cookies):
+def _headers(headers, cookie):
     result = []
     for k,v in headers.items():
         result.append((k, v))
-    for c in cookies:
-        result.append(("Set-Cookie", c))
+    for c in cookie.itervalues():
+        result.append(("Set-Cookie", c.output(header='')))
+
     return result
 
 def status(code):
@@ -70,7 +69,7 @@ class Request(object):
             self.form_data  = parse_qs(self.form_vals)
 
         self._headers   = {}
-        self._cookies   = {}
+        self._cookie    = SimpleCookie()
         self.env        = env
 
     def get(self, key):
@@ -108,38 +107,36 @@ class Request(object):
         """ Adds a header for the response """
         return self.header(key, value)
 
-    def add_cookie(self, key, value, path='/', expires=None, http_only=True, secure=False):
+    def add_cookie(self, key, value, path="/", domain=None, expires=None, http_only=True, secure=False):
         """ Adds a `Set-Cookie` header to the response """
-        #cookie = "{0}={1}; Path={2}".format(key, value, path)
-        #if expires:
-        #    cookie += "; Expires="+expires
-        #if http_only:
-        #    cookie += "; HttpOnly"
-        #if secure:
-        #    cookie += "; Secure"
-        #self._cookies.append(cookie)
-        cookie = cookies.SimpleCookie()
-        cookie[key] = value
-        c = cookie[key]
-
-        c["path"] = path
+        self._cookie[key] = value
+        c = self._cookie[key]
+        if domain is None and self.env["SERVER_NAME"] != "localhost":
+            c["domain"] = self.env["SERVER_NAME"]
+        if path is not None:
+            c["path"] = path
         if expires:
-            c["expires"] = expires
+            if type(expires) == datetime:
+                c["expires"] = expires.strftime("%d %b %Y %H:%M:%S GMT")
+            else:
+                c["expires"] = expires
         if http_only:
-            c["httponly"] = ''
+            c["httponly"] = True
         if secure:
-            c["secure"] = ''
+            c["secure"] = True
 
-        self._cookies[key] = cookie
-
-    def del_cookie(self, key):
+    def del_cookie(self, key, path="/", domain=None, expires=None, http_only=True, secure=False):
         """ Sets a cookie to expire """
-        cookie = cookies.SimpleCookie()
-        cookie.load(self.get("HTTP_COOKIE"))
-        if key in cookie:
-            c = cookie[key]
-            c["expires"] = "Thu, 01 Jan 1970 00:00:00 GMT"
-            self._cookies[key] = cookie
+        http_cookie = self.env.get("HTTP_COOKIE")
+
+        if not http_cookie:
+            return
+
+        c = SimpleCookie()
+        c.load(http_cookie)
+        if key in c:
+            self._cookie[key] = "deleted"
+            self._cookie[key]["expires"] = "Thu, 01 Jan 1970 00:00:01 GMT"
 
     def set_response_code(self, code):
         """ Sets the response code for the request """
@@ -164,7 +161,5 @@ class Request(object):
         if self._resp_code:
             code = self._resp_code
 
-        cookies = [str(c) for c in self._cookies.values()]
-
         self._resp_started = True
-        self._resp(status(code), _headers(self._headers, cookies))
+        self._resp(status(code), _headers(self._headers, self._cookie))
